#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2002-2005 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: bookmark.py 391 2005-05-01 21:19:37Z vslavik $
#
#  Bookmarking functions
#

import sys, string, os, xml.dom.minidom
import utils
import book #NOTE: books must be loaded FIRST
from xmlutils import *

bookmarks = {}

class Bookmark:
    def __init__(self, name="", book="", url=""):
        self.name = name
        self.book = book
        self.url = url
        
    def getFullUrl(self):
        return "http://localhost:%i/%s" % (utils.getServerPort(), self.book.title + self.url)

def getBookmarksForBook(bookname):
    bmList = []
    for bm in bookmarks:
        if bookmarks[bm].book.title == bookname: 
            bmList.append(bm)
            
    return bmList

def loadBookmarks():
    global bookmarks
    myfile = os.path.join(utils.getConfigDir(), "bookmarks.xml")
    if os.path.exists(myfile):
        doc = xml.dom.minidom.parse(myfile)
    
        nodebookm = xmlGetNode(XML_NS_BOOKMARKS, doc, 'bookmarks')
        if nodebookm:
            for node in nodebookm.getElementsByTagNameNS(XML_NS_BOOKMARKS, 'bookmark'):
                burl = node.getAttribute('url')
                bbook = book.books[node.getAttribute('book')]
                btitle = xmlGetText(node)
                bookmarks[btitle] = Bookmark(btitle, bbook, burl)

def saveBookmarks():
    doc = xml.dom.minidom.Document()
    root = xmlMakeNode(XML_NS_BOOKMARKS, doc, None, "bookmarks")
    for bm in bookmarks.keys():
        bmnode = xmlMakeNode(XML_NS_BOOKMARKS, doc, root, "bookmark", bm)
        bmnode.setAttribute("book", bookmarks[bm].book.title)
        bmnode.setAttribute("url", bookmarks[bm].url)
        
    if HAVE_PYXML:
        xml.dom.ext.PrettyPrint(doc, open(os.path.join(utils.getConfigDir(), "bookmarks.xml"), "wt"))
    else:
        # keep minidom's toxml() happy, PyXML doesn't need this hack:
        root.setAttribute("xmlns", XML_NS_BOOKMARKS)
        open(os.path.join(utils.getConfigDir(), "bookmarks.xml"), "wt").write(doc.toxml())
