#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2002-2005 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: HtmlBrowser.py 415 2005-06-08 21:06:01Z vslavik $
#
#  Abstract interface to various browsers (Mozilla, MSIE, wxHTML)
#

from wxPython import *
from wxPython.wx import *
from wxPython.html import *

import sys, os, os.path
import utils

# Prefered browser:
HtmlBrowser = None

# Specific browsers:
IEBrowserClass = None
MozillaBrowserClass = None
WebKitBrowserClass = None

# -------------------------------------------------------------------------
#                             wxIEHtmlWindow
# -------------------------------------------------------------------------

if sys.platform == 'win32':
    try:
        from wx.lib.iewin import *

        class IEBrowser(IEHtmlWindow):
                
            name = 'msie'
            def __init__(self, parent, id):
                IEHtmlWindow.__init__(self, parent, id)
                self.OnPageChanged = lambda x: x
                self.OnStatusChanged = lambda x,y: x
                self.OnTitleChanged = lambda x: x
                self.OnProgress = lambda x: x
                self.url = ''
                self.title = ''
                self.backEnabled = True
                self.forwardEnabled = True
                EVT_BeforeNavigate2(self, -1, self.__OnUrlChanged)
                EVT_StatusTextChange(self, -1, self.__OnStatusChanged)
                EVT_TitleChange(self, -1, self.__OnTitleChanged)
                EVT_ProgressChange(self, -1, self.__OnProgress)
                # This would be used to track back/next button state...
                # if iewin supported it. ;-/
                #EVT_CommandStateChange(self, -1, self.__OnStateChanged)                

            def __OnUrlChanged(self, event):
                self.url = event.URL
                self.OnPageChanged(self.url)

            def __OnStatusChanged(self, event):
                self.OnStatusChanged(event.Text, False)
            
            def __OnTitleChanged(self, event):
                self.title = event.Text
                self.OnTitleChanged(event.Text)
            
            def __OnProgress(self, event):
                if event.ProgressMax > 0: # avoid division by zero error
                    self.OnProgress(float(event.Progress) /
                                    float(event.ProgressMax))

            def LoadURL(self, url):
                self.LoadUrl(url)
            def GetPageTitle(self):
                return self.GetTitle()
            def GetPageURL(self):
                return self.url
            def Reload(self):
                self.Refresh(REFRESH_COMPLETELY)
            # See above, we need to implement EVT_CommandStateChange
            # in wxIEHtmlWin.cpp to manage back/next state.
            def CanGoBack(self):
                return self.backEnabled
            def CanGoForward(self):
                return self.forwardEnabled

            def SavePrefs(self):
                pass #wxMozillaSettings_SavePrefs()

        IEBrowserClass = IEBrowser
        IEBrowserVersion = 'Internet Explorer'
     
    except:
        pass

elif sys.platform == 'darwin':
    try:
        from wxPython.webkit import *
        class WebKitBrowser(wxWebKitCtrl):
                
            name = 'webkit'
            def __init__(self, parent, id):
                wxWebKitCtrl.__init__(self, parent, id)
                self.OnPageChanged = lambda x: x
                self.OnStatusChanged = lambda x,y: x
                self.OnTitleChanged = lambda x: x
                self.OnProgress = lambda x: x
                self.url = ''
                self.title = ''
                #self.backEnabled = True
                #self.forwardEnabled = True
                EVT_WEBKIT_STATE_CHANGED(self, self.OnStateChanged)
                #EVT_BeforeNavigate2(self, -1, self.__OnUrlChanged)
                #EVT_StatusTextChange(self, -1, self.__OnStatusChanged)
                #EVT_TitleChange(self, -1, self.__OnTitleChanged)
                #EVT_ProgressChange(self, -1, self.__OnProgress)
                # This would be used to track back/next button state...
                # if iewin supported it. ;-/
                #EVT_CommandStateChange(self, -1, self.__OnStateChanged)                

            def OnStateChanged(self, event):
                self.url = event.GetURL()
                
            def GetPageURL(self):
                return self.url
            def __OnUrlChanged(self, event):
                self.url = event.URL
                self.OnPageChanged(self.url)

            def __OnStatusChanged(self, event):
                self.OnStatusChanged(event.Text, False)
            
            def __OnTitleChanged(self, event):
                self.title = event.Text
                self.OnTitleChanged(event.Text)
            
            def __OnProgress(self, event):
                if event.ProgressMax > 0: # avoid division by zero error
                    self.OnProgress(float(event.Progress) /
                                    float(event.ProgressMax))

            def SavePrefs(self):
                pass #wxMozillaSettings_SavePrefs()

        WebKitBrowserClass = WebKitBrowser
        WebKitBrowserVersion = 'WebKit (Safari)'
     
    except:
        pass
# -------------------------------------------------------------------------
#                                Mozilla
# -------------------------------------------------------------------------

try:
    from wxPython.mozilla import *
    import wxPython.mozilla
    
    # is it new enough wxMozilla?
    try:
        MOZILLA_VERSION = wxPython.mozilla.__version__
    except AttributeError:
        raise RuntimeError('Documancer needs wxMozilla >= 0.5.3')
    
    # Setting Mozilla path doesn't work if you run the documancer.py 
    # source file on Win because DOCUMANCER_HOME will be pointing to 
    # the src dir. So check to see if the Mozilla path exists before
    # adding it. This way the system Python's wxMozilla libs will be
    # used if mozpath is incorrect.
    mozpath = '%s\\win32\\mozilla' % os.environ['DOCUMANCER_HOME']
    if sys.platform == 'win32' and os.path.exists(mozpath):
        wxMozillaSettings_SetMozillaPath(mozpath)
   
    wxMozillaSettings_SetProfilePath(
                    os.path.join(utils.getConfigDir(), 'mozilla-profile'))

    class MozillaBrowser(wxMozillaBrowser):
        
        name = 'mozilla'
        def __init__(self, parent, id):
            wxMozillaBrowser.__init__(self, parent, id)
            self.OnPageChanged = lambda x: x
            self.OnStatusChanged = lambda x,y: x
            self.OnTitleChanged = lambda x: x
            self.OnProgress = lambda x: x
            self.url = ''
            EVT_MOZILLA_URL_CHANGED(self, -1, self.__OnUrlChanged)
            EVT_MOZILLA_STATUS_CHANGED(self, -1, self.__OnStatusChanged)
            EVT_MOZILLA_TITLE_CHANGED(self, -1, self.__OnTitleChanged)
            EVT_MOZILLA_PROGRESS(self, -1, self.__OnProgress)

        def __OnUrlChanged(self, event):
            self.url = event.GetNewURL()
            self.OnPageChanged(self.url)

        def __OnStatusChanged(self, event):
            self.OnStatusChanged(event.GetStatusText(), event.IsBusy())
        
        def __OnTitleChanged(self, event):
            self.OnTitleChanged(event.GetTitle())
        
        def __OnProgress(self, event):
            if event.GetTotalMaxProgress() != -1:
                self.OnProgress(float(event.GetTotalCurrentProgress()) /
                                float(event.GetTotalMaxProgress()))
            elif event.GetSelfMaxProgress() != -1:
                self.OnProgress(float(event.GetSelfCurrentProgress()) /
                                float(event.GetSelfMaxProgress()))
        
        def GetPageTitle(self):
            return self.GetTitle()
        def GetPageURL(self):
            return self.url

        def SavePrefs(self):
            wxMozillaSettings_SavePrefs()

    MozillaBrowserClass = MozillaBrowser
    MozillaBrowserVersion = 'Mozilla (wxMozilla %i.%i.%i)' % wxPython.mozilla.__version__

except ImportError:
    pass


# -------------------------------------------------------------------------
#                           wxHtmlWindow (fallback)
# -------------------------------------------------------------------------

# wxHTML is worst of all, don't bother with it if there's any other option:
if MozillaBrowserClass == None and IEBrowserClass == None and WebKitBrowserClass == None:
    
    print 'warning: using minimalistic HTML renderer (wxHTML),'
    print '         please install wxMozilla!'
    
    class wxHtmlBrowser(wxHtmlWindow):
        
        name = 'wxhtml'
        def __init__(self, parent, id):
            wxHtmlWindow.__init__(self, parent, id)
            self.OnPageChanged = lambda x: x

        def LoadURL(self, url):
            self.LoadPage(url)
        def GoBack(self):
            self.HistoryBack()
        def GoForward(self):
            self.HistoryForward()
        def GetPageTitle(self):
            return self.GetOpenedPageTitle()
        def GetPageURL(self):
            return self.GetOpenedPage()
        def CanGoBack(self):
            return self.HistoryCanBack()
        def CanGoForward(self):
            return self.HistoryCanForward()
            
        def Stop(self): pass
        def Reload(self): pass
            
        def SavePrefs(self): pass

    HtmlBrowser = wxHtmlBrowser

    import wxPython
    browserVersion = 'wxWidgets built-in (version %s)' % wxPython.wx.__version__

else:
    # choose preferred browser:
    if MozillaBrowserClass != None:
        HtmlBrowser = MozillaBrowserClass
        browserVersion = MozillaBrowserVersion
    elif IEBrowserClass != None:
        HtmlBrowser = IEBrowserClass
        browserVersion = IEBrowserVersion
    elif WebKitBrowserClass != None:
        HtmlBrowser = WebKitBrowserClass
        browserVersion = WebKitBrowserVersion
        
def create(parent, id, backend=None):
    """Creates browser instance, using provided backend, if available."""
    if backend == 'msie' and IEBrowserClass != None:
        return IEBrowser(parent, id)
    elif backend == 'mozilla' and MozillaBrowserClass != None:
        return MozillaBrowser(parent, id)
    elif backend == 'webkit' and WebKitBrowserClass != None:
        return WebKitBrowser(parent, id)
    else:
        return HtmlBrowser(parent, id)

def determineBrowser(backend):
    """Returns name of browser to use, provided the user would like to use
       specified one (which may be None or not supported by this build)."""
    if (backend == 'msie' and IEBrowserClass != None):
        return 'msie'
    if (backend == 'mozilla' and MozillaBrowserClass != None):
        return 'mozilla'
    if (backend == 'webkit' and WebKitBrowserClass != None):
        return 'webkit'
    return HtmlBrowser.name
