#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2005 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: provider.py 315 2005-02-08 21:05:39Z vslavik $
#
#  Base class for providers based on single file
#

import os
import providers

class FileBasedProvider(providers.DocumentationProvider):
        
    def __init__(self):
        # name of the attribute used to store the filename:
        self.filenameAttribute = 'filename'
    
    def getFileLabel(self):
        """Returns label shown near file selector"""
        return _('Book file:')

    def getFileDescription(self):
        """Returns description of the file (e.g. 'HTML file')"""
        raise NotImplementedError

    def getFileMasks(self):
        """Returns list of wildcards for file selector."""
        raise NotImplementedError
    
    def getConfigurationDialog(self, parent):
        import os.path
        from wxPython.wx import wxPanel, wxBoxSizer, \
                                wxTAB_TRAVERSAL, wxVERTICAL, wxALL, wxEXPAND
        from wxPython.lib.filebrowsebutton import FileBrowseButton
        import utils

        class SettingsPanel(wxPanel):
            def __init__(self, parent, label, mask, fattr):
                wxPanel.__init__(self, parent, -1, style=wxTAB_TRAVERSAL)
                self.fattr = fattr
                startdir = utils.config().Read('/GuiState/LastDir', '')
                self.file = FileBrowseButton(self, -1,
                                             labelText=label,
                                             fileMask=mask,
                                             changeCallback=self.OnFileChange,
                                             startDirectory=startdir)

                sizer_8 = wxBoxSizer(wxVERTICAL)
                sizer_8.Add(self.file, 0, wxALL|wxEXPAND, 5)
                self.SetAutoLayout(1)
                self.SetSizer(sizer_8)
                sizer_8.Fit(self)
                sizer_8.SetSizeHints(self)

            def SetDialog(self, dlg):
                self.dialog = dlg

            def OnFileChange(self, event):
                self.dialog.UpdateBook()
                utils.config().Write('/GuiState/LastDir',
                                     os.path.dirname(self.file.GetValue()))

            def SetData(self, attr):
                if self.fattr in attr:
                    self.file.SetValue(attr[self.fattr])

            def GetData(self):
                return {self.fattr: self.file.GetValue()}

        label = self.getFileLabel()
        mask = '%s|%s' % (self.getFileDescription(),
                          ';'.join(self.getFileMasks()))
        return SettingsPanel(parent, label, mask, self.filenameAttribute)
