#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2002-2005 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: provider.py 405 2005-06-02 10:06:47Z vslavik $
#
#  HTML documentation provider
#

import os.path, sys, urllib

import providers
from providers.filebased import FileBasedProvider

class HtmlProviderBase(FileBasedProvider):
    """Base class that should be used by derived providers (e.g. devhelp)
       instead of HtmlProvider -- the latter contains some backward
       compatibility hacks that are not desirable outside of it."""

    def __init__(self):
        FileBasedProvider.__init__(self)
        self.mimeTypes = {}
    
    def getFileDescription(self):
        return _('HTML files')

    def getFileMasks(self):
        return ['*.html', '*.htm']

    def _getBasedir(self, book):
        """Returns basedir attribute, possibly merged with book.definitionFile
           if it is !=None."""
        basedir = book.getAttr('basedir')
        if book.definitionFile != None:
            return os.path.join(os.path.dirname(book.definitionFile), basedir)
        else:
            return basedir

    def getID(self):
        return 'html'
    
    def getName(self):
        return _('HTML')

    def getDescription(self):
        return _('Documentation in HTML format.')

    def serve(self, book, url):
        if url == '':
            return None
        url = self._url2filename(url, book)
        try:
            if os.path.isdir(url):
                nm = os.path.join(url, "index.html")
                if not os.path.isfile(nm):
                    nm = os.path.join(url, "index.htm")
                if os.path.isfile(nm):
                    url = nm
            f = open(url, 'rb')
            data = f.read()
            f.close()
            if not self.mimeTypes.has_key(url):
                uo = urllib.urlopen(urllib.quote(url))
                self.mimeTypes[url] = uo.info().gettype()
                uo.close()
            return (data, self.mimeTypes[url])
            
        except IOError, e:
            return None
    
    def getHomeURL(self, book):
        return self._filename2url(book.getAttr(self.filenameAttribute), book)
    
    def getURLForIndexing(self, book):
        return self.getHomeURL(book)

    def normalizeBookPaths(self, abook):
        FileBaseProvider.normalizeBookPaths(self, abook)
        if abook.attr.has_key('basedir'):
            basedir = abook.attr['basedir']
            if not os.path.isabs(basedir):
                rootdir = os.path.dirname(abook.getDefinitionFile())
                abook.attr['basedir'] = os.path.join(rootdir, basedir)

    # helpers:
    def _filename2url(self, filename, book):
        if sys.platform == 'win32':
            filename = filename.replace('\\','/')
            if (len(filename) > 0 and filename[1] == ':' and
                not book.hasAttr('basedir')):
                filename = '/%s%s' % (filename[0], filename[2:])
        return urllib.quote(filename)

    def _url2filename(self, url, book):
        fname = urllib.unquote(url)
        if fname[0] != "/": fname = "/" + fname
        if sys.platform == 'win32' and not book.hasAttr('basedir'):
            fname = '%s:%s' % (fname[1], fname[2:])
        if book.hasAttr('basedir'):
            fname = os.path.join(self._getBasedir(book), fname[1:])
        return fname


class HtmlProvider(HtmlProviderBase):
    def __init__(self):
        HtmlProviderBase.__init__(self)
        self.filenameAttribute = 'rootfile' # for backward compatibility
    
    def getFileLabel(self):
        return _('Title page:')
 

providers.register(HtmlProvider())
