#
#  This file is part of Documancer (http://documancer.sf.net)
#
#  Copyright (C) 2005 Vaclav Slavik
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  $Id: provider.py 329 2005-02-12 18:41:38Z vslavik $
#
#  WWW-based books
#

import sys
import urlparse
import providers

from yahoo.search import webservices

class WebProvider(providers.DocumentationProvider):

    def getID(self):
        return 'web'
    
    def getName(self):
        return _('WWW site')
    
    def getDescription(self):
        return _("""Plugin for reading web pages.<br>
                    Powered by Yahoo! Search.""")

    def serve(self, book, url):
        if url == 'indexing':
            return ('', 'text/html')
        else:
            reloc = book.getAttr('url')
            return (reloc, 'REDIRECT')

    def getHomeURL(self, book):
        return 'home'
    
    def getURLForIndexing(self, book):
        return 'indexing'

    def getConfigurationDialog(self, parent):
        from confgui import WebSettingsPanel
        return WebSettingsPanel(parent)

    def search(self, book, query):
        site = urlparse.urlparse(book.getAttr('url'))[1]
        
        srch = webservices.create_search('web', 'documancer')
        srch.query = '%s site:%s' % (query, site)
        srch.results = 20
        
        try:
            results = srch.parse_results()
            ret = []
            for r in results:
                ret.append((r.Title, r.Url, ''))
            return ret
        except Exception, err:
            from indexers import QueryParsingError
            raise QueryParsingError(query, str(err))

providers.register(WebProvider())
